///////////////////////////////////////////////////
//(c)2002 Erik Sjolander
//This code may be used and modified for non-profit, 
//educational purposes, as long as this copyright
//notice is preserved.  Any other use requires my 
//permission.  
//
//demo.c
//The main file for running my robot.  
///////////////////////////////////////////////////

#include <io.h>
#include <interrupt.h>

#include "global.h"
#include "motor.h"
#include "a_d.h"
#include "sonar.h"
#include "red_button.h"

#include "demo.h"

#include "obstacle_avoidance.h"
#include "line_following.h"
#include "bump_switches.h"
#include "glass_filling.h"

//#include "uart.h"

s16 ad_data[8];
s16 ad_offset[8];
s16 ad_data_scaled[8];
u16 sonar;

void delay(u16 delay_time) {
    do {
      u08 i=0;
      do {
      asm volatile("nop\n\t"
          "nop\n\t"
          "nop\n\t"
          "nop\n\t"
          ::);
      } while(--i);
    } while(--delay_time);
}


void get_ad_scaled(void)
{
    get_all_ad_avg(ad_data);
    u08 i;
    for(i=0; i<8; i++) {
        ad_data_scaled[i] = ad_data[i]-ad_offset[i];
    }

}

void set_ad_scale(void) 
{
    get_all_ad_avg(ad_offset);
}


/*
//doesn't work - I don't know why
u08 get_max_index(struct behavior in[])
{
    u08 i;
    s08 confidence=-100;
    u08 index = 0;
    for(i=0;i<behavior_array_length; i++) {
        if(in[i].confidence > confidence) {
            confidence = in[i].confidence;
            index = i;
        }
    }
    
    return index;
}
*/


int main(void)
{
    DDRA = 0;
    DDRB = 0; 
    DDRC = 0;
    DDRD = 0;
    
    PORTC = 0x30; //turn off the LEDs
    DDRC  = 0x30;  //Turn on high portC led's

    /*
    char out[32];
    uart_init();
    */
    motor_init();
    ad_init();
    sonar_init();
    red_button_init();

    
    glass_filling_init();
    obstacle_avoidance_init();
    line_following_init();
    bump_switches_init();
    
    sei();
    
    
    struct behavior main_behavior[behavior_array_length];

    wait_for_button();
    set_ad_scale();
    
    wait_for_button();
    
    sonar_start(); //get sonar reading for first time through loop

    //u08 portc_led = 0x01;
    
    u08 index;
    
        
    for (;;)                        // loop forever 
    {
        sonar = sonar_closest(); //get the sonar data 
        sonar_start();        //start a new sonar ping
        get_ad_scaled();

        line_following(&main_behavior[0]);
        obstacle_avoidance(&main_behavior[1]);
        bump_switches(&main_behavior[2]);
        glass_filling(&main_behavior[3]);
        
        //index = get_max_index(main_behavior);
        u08 i;
        index =  0;
        s08 confidence = -100;
        for(i=0; i<behavior_array_length; i++) {
           if(main_behavior[i].confidence > confidence) {
               confidence = main_behavior[i].confidence;
               index = i;
           }
        }
        
        motor_speed(main_behavior[index].left_motor, main_behavior[index].right_motor);
        set_acceleration(main_behavior[index].acceleration);
        //portc_led++;
        //PORTC = ((~portc_led)&0xF0) | (PORTC & 0x0F);
        PORTC = ( (~(index<<4)) & 0x30 ) | (PORTC & 0x0F);
        delay(175); //about 1/18 sec.

        /*
        sprintf(out,"line: left: %i\tr:%i\tc:%i",
              main_behavior[0].left_motor, main_behavior[0].right_motor, main_behavior[0].confidence);;
        uart_putstr(out);
        sprintf(out,"obst: left: %i\tr:%i\tc:%i",
              main_behavior[1].left_motor, main_behavior[1].right_motor, main_behavior[1].confidence);;
        uart_putstr(out);
        delay(0x2000);
        */
            
    }

       
}


