///////////////////////////////////////////////////
//(c)2002 Erik Sjolander
//This code may be used and modified for non-profit, 
//educational purposes, as long as this copyright
//notice is preserved.  Any other use requires my 
//permission.  
//
//line_following.c
//A line-following algorithm, based on 3 analog
//sensors at the front of my robot.  
///////////////////////////////////////////////////
#include "demo.h"
#include "global.h"
#include <io.h>
#include "motor.h"
#include "line_following.h"

s16 line_difference(void) 
{
    return (ad_data[left_line_sensor] - ad_data[right_line_sensor]);

}

void line_following_init()
{
    ;
}

void line_following(struct behavior* out) 
{
    static s16 integral_error=0;
    static u08 have_line = 0;
    s16 left_tmp, right_tmp;
    s16 line_delta;
    s08 confidence;
        
    line_delta = line_difference();
       
    //diff = abs_val(left_tmp - right_tmp); //current wheel speed difference
    if(have_line) {
        integral_error += line_delta/32;
        left_tmp = MOTOR_FORWARD_SLOW - integral_error - line_delta;
        right_tmp = MOTOR_FORWARD_SLOW + integral_error + line_delta;
    } else {
        left_tmp = MOTOR_FORWARD_SLOW - line_delta;
        right_tmp = MOTOR_FORWARD_SLOW + line_delta;    
    }
 
    if(left_tmp > MOTOR_TOP) {
        left_tmp = MOTOR_TOP;
    }else if(left_tmp < -MOTOR_TOP) {
        left_tmp = -MOTOR_TOP;
    } 
    if(right_tmp>MOTOR_TOP) {
        right_tmp = MOTOR_TOP;
    } else if(right_tmp < -MOTOR_TOP) {
        right_tmp = -MOTOR_TOP;
    }
    
    if( (ad_data[center_line_sensor]-ad_data[left_line_sensor] > 200) &&
          (ad_data[center_line_sensor]-ad_data[right_line_sensor] > 200)) {
        confidence = 25; //we *know* we're on the line      
        have_line=1;
    } else if(ad_data_scaled[center_line_sensor] > (200 - 150*have_line)) {
        confidence = 10; //maybe it's a line - center sensor is dark
        have_line = 1;
        //confidence += have_line * 10; //more important to stay on line
        //integral_error = integral_error*have_line;
    } else {
        confidence = 0;  //no line :-(
        integral_error = 0;
        have_line = 0;
    }
    
    out->left_motor = left_tmp;
    out->right_motor = right_tmp;
    out->confidence = confidence;
    out->acceleration = ACCELERATION_FAST;
    
}
