///////////////////////////////////////////////////
//(c)2002 Erik Sjolander
//This code may be used and modified for non-profit, 
//educational purposes, as long as this copyright
//notice is preserved.  Any other use requires my 
//permission.  
//
//obstacle_avoidance.c
//Basic obstacle avoidance code.  Assumes the presence
//of variables set in demo.c  
///////////////////////////////////////////////////
#include "demo.h"
#include "global.h"
#include <io.h>
#include "motor.h"
#include "obstacle_avoidance.h"

#define LEFT 1
#define RIGHT 0

s16 max(s16 a, s16 b) 
{
    return a>b?a:b;
}

void obstacle_avoidance_init() 
{
   ;
}

u08 random(void) 
{
    u08 rand = TCNT1L ^ TCNT0; //the two counters I'm using
    rand *= 71; //multiply by a prime to mix the bits around
    return rand;
}
/*
void turn(u08 direction, u08 time) 
{
    if((direction & 0x1) == right) {
        //turn right
    } else {
        //turn left
    }
}
*/
void obstacle_avoidance(struct behavior* out)
{
    s16 right_motor, left_motor;
    s08 confidence;
    s16 right = ad_data[right_eye];
    s16 left = ad_data[left_eye];
    static s08 count=0;
    static u08 direction;
    
    if(count == 0) {
        if(right<=150 && left<=150 && sonar>=17) {
            //don't see anything
            if(sonar > 24) {
                right_motor = MOTOR_FORWARD_FAST;
                left_motor = MOTOR_FORWARD_FAST;
            } else {
                right_motor = MOTOR_FORWARD_SLOW;
                left_motor = MOTOR_FORWARD_SLOW;
            }
            confidence = 1;
        } else {
            //there's something out there...
            if(right > (left + left/4) ) {
                //see something in right eye (to left of bot)
                right_motor = MOTOR_BACKWARD_SLOW;
                left_motor = MOTOR_FORWARD_SLOW;
                direction = RIGHT;    
            } else if(left > (right + right/4)) {
                // see something to the right of the bot
                right_motor = MOTOR_FORWARD_SLOW;
                left_motor = MOTOR_BACKWARD_SLOW;
                direction = LEFT;
            } else {
                //pretty much in front of us
                //turn in a random direction
                if(random() & 0x01) {
                    right_motor = MOTOR_BACKWARD_SLOW;
                    left_motor = MOTOR_FORWARD_SLOW;
                    direction = RIGHT;
                } else {
                    right_motor = MOTOR_FORWARD_SLOW;
                    left_motor = MOTOR_BACKWARD_SLOW;
                    direction = LEFT;
                }
            }
            confidence = 11;
            count = ((random()&0x03) + 1)*16;
        }
        
    } else { 
        //we're in the middle of turning
        count--;
        if(direction == RIGHT) {
            right_motor = MOTOR_BACKWARD_SLOW;
            left_motor = MOTOR_FORWARD_SLOW;
        } else {
            right_motor = MOTOR_FORWARD_SLOW;
            left_motor = MOTOR_BACKWARD_SLOW;
        }
        confidence = 11;    
    }
    
    //nothing
    // eyes <= 150
    // sonar >= 17
    
    //kinda see something
    // "eyes" > 150
    // sonar < 17
    
    //really need to get out of here
    // "eyes" > 250
    // sonar < 10
    
/*
    if(ad_data[right_eye] < 100) {
        right_motor = MOTOR_FORWARD_FAST;
        confidence = 1; 
    } else if(ad_data[right_eye] < 150) {
        confidence = 5;
        right_motor = MOTOR_FORWARD_SLOW;
    } else if(ad_data[right_eye] < 300) {
        right_motor = MOTOR_STOP;
        confidence = 11;
    } else {
        right_motor = MOTOR_BACKWARD_SLOW;
        confidence = 50;
    }

    if(ad_data[left_eye] < 100) {
        left_motor = MOTOR_FORWARD_FAST;
    } else if(ad_data[left_eye] < 150) {
        left_motor = MOTOR_FORWARD_SLOW;
        confidence = max(confidence, 5);
    } else if(ad_data[left_eye] < 300) {
        left_motor = MOTOR_STOP;
        confidence = max(confidence, 11);
    } else {
        left_motor = MOTOR_BACKWARD_SLOW;
        confidence = max(confidence, 50);
    }
*/
    /*
    if(left_motor == MOTOR_STOP && right_motor == MOTOR_STOP) {
        left_motor = MOTOR_FORWARD_SLOW;
        right_motor = MOTOR_BACKWARD_SLOW;
    }
    */
    out->right_motor = right_motor;
    out->left_motor = left_motor;
    out->confidence = confidence;    
    out->acceleration = ACCELERATION_SLOW;
    
} 
