///////////////////////////////////////////////////
//(c)2002 Erik Sjolander
//This code may be used and modified for non-profit, 
//educational purposes, as long as this copyright
//notice is preserved.  Any other use requires my 
//permission.  
//
//red_button.c
//Implements a pause routine, and wait-until-button-
//pressed, using a switch on PD2 (INT0).  
///////////////////////////////////////////////////
#include <interrupt.h>
#include <sig-avr.h>
#include <io.h>
#include <inttypes.h>
#include "global.h"
#include "red_button.h"

//pause/unpause
//wait_for_button

volatile s08 waiting;

//delay for about 200ms
void short_delay(void) 
{
    int32_t tmp = 400000;
    do{;}while(--tmp);
}

//Pause function: press once to pause, 
//press again to unpause
SIGNAL(SIG_INTERRUPT0)      
{ 
    PORTB &= ~(_BV(1)); // turn on D9 diode
    if(waiting) {
        //we are waiting for a button press
        //make sure button is low
        short_delay();
        //wait for release of button
        loop_until_bit_is_set(PIND, 2);
        //wait for pin to stabilize
        short_delay();
        waiting = 0;
    } else {
        //act as pause function
        s16 ocr1a_value = OCR1A;
        s16 ocr1b_value = OCR1B;
        OCR1A = 0;
        OCR1B = 0;
        s08 ocr0_value = OCR0;
        OCR0 = 0;
        
        //wait for button release
        short_delay();
        loop_until_bit_is_set(PIND, 2);
        short_delay();
        
        //wait for button press
        loop_until_bit_is_clear(PIND,2);
        short_delay();
        //wait for button release
        loop_until_bit_is_set(PIND, 2);
        short_delay();
        
        //restore everything
        OCR1A = ocr1a_value;
        OCR1B = ocr1b_value;
        OCR0  = ocr0_value;
    }
    PORTB |= _BV(1);  //turn off D9 diode
}


void red_button_init(void)
{
    waiting = 0;
    
    SFIOR &= ~(_BV(PUD)); // enable pull-up resistors
    DDRD &= ~(_BV(2)); // pin2 is input
    PORTD |= _BV(2);   //pull-up resistor is on
    
    PORTB |= _BV(1);   //use D9 diode as pause status light
    DDRB |= _BV(1);
    
    
    MCUCR &= ~(_BV(ISC11)|_BV(ISC10));  //INT0 is low-level triggered
    GIFR |= _BV(INTF0); // clear the flag
    //PORTB &= ~(_BV(1));
    short_delay();  //make sure everything has stabilized
    //PORTB |= _BV(1);
    GICR |= _BV(INT0); //enable the interrupt
}

void wait_for_button(void)
{
    waiting = 1;
    do{;}while(waiting);
}
