///////////////////////////////////////////////////
//(c)2002 Erik Sjolander
//This code may be used and modified for non-profit, 
//educational purposes, as long as this copyright
//notice is preserved.  Any other use requires my 
//permission.  
//
//i2c_test.c
//Tests the Devantech SRF08 I2C sonar module.  
///////////////////////////////////////////////////

#include <io.h>
#include <interrupt.h>
#include "global.h"
#include "uart.h"
#include "sonar.h"


void delay(u16 delay_time) {
    do {
      u08 i=0;
      do {
      asm volatile("nop\n\t"
          "nop\n\t"
          "nop\n\t"
          "nop\n\t"
          ::);
      } while(--i);
    } while(--delay_time);
}


int main(void)
{
    DDRA = 0;    //all ports are inputs
    DDRB = 0;
    DDRC = 0; 
    DDRD = 0;
    
    //u08 data[36]; //sonar data
    u16 ranges[17];
    u08 light_sensor=0;
    char out[64];
    
    delay(0x2000); //give bootloader a second to reset
    
    uart_init();
    sonar_init();
    sei(); 
    
    u16 closest=0;
    s08 error1, error2;
    u08 i;
    
    for(;;) {
        error1 = sonar_start();
        error2 = sonar_data(ranges, &light_sensor);
        closest = sonar_closest();
        
        if(error1==0 && error2==0) {
            for(i=0; i< 17; i++) {
                if(i%8 == 0) {
                    uart_nl();
                }
                sprintf(out, " %i:%i ",i,ranges[i]);
                uart_putstr(out);
            }
            uart_nl();
        }
        sprintf(out, "closest: %i\tlight: %i\r\nerror1: %i error2: %i", 
              closest, light_sensor, error1, error2);
        uart_putstr(out);
        uart_nl();
        delay(0x2000);
    }
    
}


