///////////////////////////////////////////////////
//(c)2002 Erik Sjolander
//This code may be used and modified for non-profit, 
//educational purposes, as long as this copyright
//notice is preserved.  Any other use requires my 
//permission.  
//
//a_d.c
//a/d routines.  Includes 4-sample averaging.
///////////////////////////////////////////////////

#include <io.h>
#include "global.h"
#include "a_d.h"

void ad_init(void)
{
    DDRA  = 0x00;  //all A ports are inputs
    ADMUX = 0x00;  //select AD0, external Vref, not left adjusted 
    ADCSR = _BV(ADEN)|_BV(ADIF)|_BV(ADPS2)|_BV(ADPS0);  //enable, clear int. flag, set ad clock to CK/32
}


s16 get_sample(u08 channel)
{
    u08 temp = ADMUX & 0xE0;  //keep top 3 bits (REFS1:0, ADLAR)
    channel &= 0x07;  //throw away top bits
    ADMUX = channel | temp;  // put two parts back together
    while(ADCSR & _BV(ADSC)){;} //wait for bit to clear - should never be set
    ADCSR |= _BV(ADSC);  //start conversion
    while(ADCSR & _BV(ADSC)){;} //wait for conversion to complete
    s16 retval = ADCW;
    return retval;
}

void get_all_ad(s16* ad_array) 
{
    u08 i;
    
    for(i=0;i<8;i++)
    {
        ad_array[i] = get_sample(i);
    }

}

void get_all_ad_avg(s16* ad_array)
{
    u08 i,j;
    get_all_ad(ad_array);
    for(i=1;i<ad_average_length;i++) {
        for(j=0;j<8;j++) {
            ad_array[j] += get_sample(j);
        }
    }
    for(i=0;i<8;i++) {
        ad_array[i] /= ad_average_length;
    }
}
/*
void delay(u16 delay_time) {
    do {
      u08 i=0;
      do {
      asm volatile("nop\n\t"
          "nop\n\t"
          "nop\n\t"
          "nop\n\t"
          ::);
      } while(--i);
    } while(--delay_time);
}

int main(void)
{
    DDRA = 0;    //all ports are inputs
    DDRB = 0;
    DDRC = 0; 
    DDRD = 0;
    
    uart_init();
    ad_init();
    
    //s16 analog7;
    char out[32];
    s16 ad_values[8];
    sei();  
    //uart_putstr("AD conversion started...\r\n");

    
    for(;;)
    {
        get_all_ad(ad_values);
        
        u08 i;
        for(i=0; i<8; i++)
        {
            //ad_values[i] = get_sample(i);
            sprintf(out, "AD%i:\t%i\r\n", i, ad_values[i]);
            uart_putstr(out);
        }
        //analog7 = get_sample(7);
        //sprintf(out, "AD7:\t%i\r\n", analog7);
        //uart_putstr(out);
        delay(0x1FFF);
        uart_nl();
    }
}
*/

