///////////////////////////////////////////////////
//(c)2002 Erik Sjolander
//This code may be used and modified for non-profit, 
//educational purposes, as long as this copyright
//notice is preserved.  Any other use requires my 
//permission.  
//
//motor.c
//Motor driver routines.  Uses the Timer1 to do
//PWM.  External TI L293D chip used.  Acceleration
//limiting is implemented.  
///////////////////////////////////////////////////
#define RIGHT_BACKWARD _BV(4)
#define RIGHT_FORWARD  _BV(5)
#define RIGHT_OCR      OCR1B

#define LEFT_BACKWARD _BV(6)
#define LEFT_FORWARD  _BV(7)
#define LEFT_OCR      OCR1A


#include <io.h>
#include <interrupt.h>
#include <sig-avr.h>

#include "global.h"
#include "motor.h"

#define PWM1_TOP 1023
#define MOTOR_START 512 //lowest PWM value that (might) start the motors.
//#define LEFT_MOTOR_OFFSET -10
//#define RIGHT_MOTOR_OFFSET -50


// left motor=OC1A
// right motor = OC1B

//range from -511 to 511
//set these to change motor speed
volatile s16 left_motor_speed;
volatile s16 right_motor_speed;
volatile s08 acceleration;

SIGNAL(SIG_OVERFLOW1)      
{
    static s08 count = 6;
    static s16 left_motor = 0;
    static s16 right_motor = 0;
    if(--count == 0) {
        if(left_motor != left_motor_speed) {
            if(left_motor < left_motor_speed) {
                //speed up the motor
                left_motor++;
            } else {
                //must need to slow down the motor
                left_motor--;
            }
            s16 left_tmp; 
            if(left_motor <0) {
                left_tmp = (-left_motor) + MOTOR_START;
            } else if(left_motor > 0) {
                left_tmp = left_motor + MOTOR_START;
            }
            LEFT_OCR = left_tmp;
        }
        if(right_motor != right_motor_speed) {
            if(right_motor < right_motor_speed) {
                //speed up the motor
                right_motor++;
            } else if(right_motor > right_motor_speed) {
                //slow down the motor
                right_motor--;
            }
            s16 right_tmp;
            if(right_motor<0) {
                right_tmp = (-right_motor) + MOTOR_START;
            } else if(right_motor>0) {
                right_tmp = right_motor + MOTOR_START;
            }
            RIGHT_OCR = right_tmp;
        }
        
        u08 control_vals = 0;
        if(right_motor < 0) {
            control_vals |= RIGHT_BACKWARD;
        } else if(right_motor > 0) {
            control_vals |= RIGHT_FORWARD;
        }
        
        if(left_motor < 0) {
            control_vals |= LEFT_BACKWARD;
        } else if(left_motor > 0) {
            control_vals |= LEFT_FORWARD;
        }
        u08 tmp = PORTB & 0x0F; //throw away top bits - these are the ones used for motor direction
        PORTB = control_vals | tmp;
        count = acceleration;             //start the counter over
    }
}

void motor_init(void)
{    PORTC = 0x30; //turn off the LEDs    PORTC = 0x30; //turn off the LEDs
    DDRD  |= _BV(5);				//Set OC1A as output
    PORTD &= ~_BV(5);				//Turn off OC1A
    DDRD  |= _BV(4);				//OC1B is output
    PORTD &= ~_BV(4);
    
    OCR1A  = 0x0000;				//Make sure pin stays off
    OCR1B  = 0x0000;
    
    TCCR1A = _BV(PWM11)|_BV(PWM10)|_BV(COM1A1)|_BV(COM1B1); //10-bit PWM, non-inverted
    TCCR1B = _BV(CS10);				//Divide clock by 1
    
    DDRB   |= 0xF0;                      //use high bits of portB to control direction
    PORTB  &= ~0xF0;                     //Turn off the outputs.  
    
    left_motor_speed = 0;
    right_motor_speed = 0;
    acceleration = ACCELERATION_SLOW;
    
    TIFR |= _BV(TOV1); //clear TCNT1 overflow flag
    TIMSK |= _BV(TOIE1);  //enable TCNT1 overflow interrupt
}


void set_acceleration(s08 _acceleration)
{
    acceleration = _acceleration;
}

//speeds are neg. for reverse
//range from -511 to 511.  0 is stopped.
void motor_speed(s16 left_motor, s16 right_motor)
{
    if(left_motor > MOTOR_TOP) {
        left_motor = MOTOR_TOP;
    } else if(left_motor < -MOTOR_TOP) {
        left_motor = -MOTOR_TOP;
    }
    
    if(right_motor > MOTOR_TOP) {
        right_motor = MOTOR_TOP;
    } else if(right_motor < -MOTOR_TOP) {
        right_motor = -MOTOR_TOP;
    }
    
    left_motor_speed = left_motor;
    right_motor_speed = right_motor;
/*
    u08 temp = PORTB;
    temp &= 0x0F;  // throw away the top bits - 
    u08 control_vals=0;
    

    if(left_motor==0) {
        LEFT_OCR = 0; 
    } 
    else {
        if(left_motor <0) {
            control_vals |= LEFT_BACKWARD;
            left_motor = -left_motor;
        } 
        else {
            control_vals |= LEFT_FORWARD;
        }
        left_motor += MOTOR_START;
        //left_motor += LEFT_MOTOR_OFFSET; //correct for different motor speeds
        if(left_motor > PWM1_TOP) {
             left_motor = PWM1_TOP;
        }
        LEFT_OCR = left_motor;
    }
    
    if(right_motor==0) {
        RIGHT_OCR = 0;
    } 
    else {
        if(right_motor<0) {
            control_vals |= RIGHT_BACKWARD;
            right_motor = -right_motor;
        } 
        else {
            control_vals |= RIGHT_FORWARD;
        }
        right_motor += MOTOR_START;
        //right_motor += RIGHT_MOTOR_OFFSET;
        if(right_motor > PWM1_TOP) {
            right_motor = PWM1_TOP;
        }
        RIGHT_OCR = right_motor;
    }
    
    PORTB = control_vals | temp;
*/
}
